/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "Map.h"

using namespace std;
using namespace FD;

class CvfdhSmooth;
   
DECLARE_NODE(CvfdhSmooth)
/*Node
 *
 * @name CvfdhSmooth
 * @category FDOPENCV_HARK:Filter
 * @description CvfdhSmooth
 *
 * @input_name IMAGEIN
 * @input_type any
 * @input_description Input image. The type should be CvfdImage or Map<int, ObjectRef>.
 *
 * @output_name IMAGEOUT
 * @output_type any
 * @output_description Output image. The type should be CvfdImage or Map<int, ObjectRef>.
 *
 * @parameter_name FILTER_TYPE
 * @parameter_type string
 * @parameter_value BLUR
 * @parameter_list BLUR:GAUSSIAN:MEDIAN:BILATERAL
 * @parameter_description 
 *
 END*/
   
   
class CvfdhSmooth : public BufferedNode {

  int m_imageInID;
  int m_imageOutID;
  string filter_type;
      
public:
  CvfdhSmooth(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    m_imageInID = addInput("IMAGEIN");
    m_imageOutID = addOutput("IMAGEOUT");
    filter_type = object_cast<String>(parameters.get("FILTER_TYPE"));
    inOrder = true;
  }      
      
  void calculate(int output_id, int count, Buffer &out)
  {

    ObjectRef in = getInput(m_imageInID, count);

    // -------------------------------------------------------------

    if (typeid(*in) == typeid(CvfdImage)) {

      RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);   
      if(imagePtr->getImage() == NULL){
	(*(outputs[m_imageOutID].buffer))[count] = ObjectRef(imagePtr);
	return;
      }

      IplImage* cv_image;
      cv_image = cvCloneImage(imagePtr->getImage());
    

      IplImage *dst_img;
  
      dst_img = cvCloneImage(cv_image);

      if(filter_type == "BLUR")
	cvSmooth (cv_image, dst_img, CV_BLUR, 5, 0, 0, 0);
      if(filter_type == "GAUSSIAN")
	cvSmooth (cv_image, dst_img, CV_GAUSSIAN, 11, 0, 0, 0);
      if(filter_type == "MEDIAN")
	cvSmooth (cv_image, dst_img, CV_MEDIAN, 5, 0, 0, 0);
      if(filter_type == "BILATERAL")
	cvSmooth (cv_image, dst_img, CV_BILATERAL, 80, 80, 0, 0);
    
      IplImage* img_out;
      img_out = cvCloneImage(dst_img);

      CvfdImage* image= new CvfdImage(img_out);  
         
      (*(outputs[m_imageOutID].buffer))[count] = ObjectRef(image); 

      cvReleaseImage (&cv_image);
      cvReleaseImage (&dst_img);
      cvReleaseImage (&img_out);
	
      // -------------------------------------------------------------

    }else if (typeid(*in) == typeid(Map<int, ObjectRef>)) {

      RCPtr<Map<int, ObjectRef> > imageMapPtr = getInput(m_imageInID, count);
      RCPtr<Map<int, ObjectRef> > outputMapPtr(new Map<int, ObjectRef>);
      
      Map<int, ObjectRef>::iterator it = imageMapPtr->begin();
      for (; it != imageMapPtr->end(); ++it) {
        CvfdImage& imagePtr = object_cast<CvfdImage>(it->second);
	IplImage* cv_image;
	cv_image = cvCloneImage(imagePtr.getImage());

	IplImage *dst_img;
  
	dst_img = cvCloneImage(cv_image);
	
	if(filter_type == "BLUR")
	  cvSmooth (cv_image, dst_img, CV_BLUR, 5, 0, 0, 0);
	if(filter_type == "GAUSSIAN")
	  cvSmooth (cv_image, dst_img, CV_GAUSSIAN, 11, 0, 0, 0);
	if(filter_type == "MEDIAN")
	  cvSmooth (cv_image, dst_img, CV_MEDIAN, 5, 0, 0, 0);
	if(filter_type == "BILATERAL")
	  cvSmooth (cv_image, dst_img, CV_BILATERAL, 80, 80, 0, 0);
	
	IplImage* img_out;
	img_out = cvCloneImage(dst_img);

	CvfdImage* image= new CvfdImage(img_out);  
	(*outputMapPtr)[it->first] = image;

	cvReleaseImage (&cv_image);
	cvReleaseImage (&dst_img);
	cvReleaseImage (&img_out);

      }
      (*(outputs[m_imageOutID].buffer))[count] = outputMapPtr; 
    }

  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdhSmooth)
};
